# coffeelint: disable=max_line_length

# require dependencies
_ = require "lodash"
EventEmitter = require("events").EventEmitter
helpers = require './helpers'
logger = require("../../shared_modules/logger")

class Producer extends EventEmitter
  constructor: (@name, @stopIntervalTime) ->
    @cache = {}
    @status = ""
    @subscriptionTime = new Date()

    @debug = ->
      args = Array.prototype.slice.call(arguments)
      args.unshift("#{@name}: ")
      args.push "(producer class)"
      logString = args.join " "
      logger.debug.apply logger, [logString]

    @info = ->
      args = Array.prototype.slice.call(arguments)
      args.unshift("#{@name}: ")
      args.push "(producer class)"
      logString = args.join " "
      logger.info.apply logger, [logString]

    @error = ->
      args = Array.prototype.slice.call(arguments)
      args.unshift("#{@name}: ")
      args.push "(producer class)"
      logString = args.join " "
      logger.error.apply logger, [logString]

    @info "Creating producer #{@name}"

    @subscribeTo = (producer, callback) =>
      process.nextTick =>
        @info @name, "Subscribing to #{producer}"
        @emit "subscribes", producer, callback

    @requestData = (reqObject, callback) =>
      {producer, action, args} = reqObject
      @info "Requesting data from #{producer} (#{action})"
      @emit "request", producer, action, JSON.stringify(args), (err, status, data) =>
        # @debug "requested data and got", err, status, data
        if data
          try
            data = JSON.parse data
          catch e
            @error "Data was not stringified!"
            @error e
            data = ""
        callback err, status, data

    @checkSubscriptionState = =>
      return unless @subscriptionTime
      diffTime = new Date() - @subscriptionTime
      if diffTime > @stopIntervalTime
        @info "Noone subscribed anymore, calling 'stop'"
        unless @stop
          @debug "I don't have a stop method!"
          return
        @stop()
        @subscriptionTime = false

    @stopInterval = false
    if @stopIntervalTime
      @subscriptionTime = new Date()
      @info "Will stop if there are no subscribers for #{@stopIntervalTime / 1000} seconds"
      @stopInterval = setInterval @checkSubscriptionState, 2000

    @getStatus = (callback) =>
      if callback
        if @cache
          @debug "Sending from cache"
          callback null, @cache
        unless @cache
          @debug "Has nothing in cache"
          callback null, ""
      else
        @debug "No callback specified"

    @publishDiff = (data) =>
      diff = helpers.getChanges @cache, data
      @cache = _.merge @cache, data if diff
      status = @status or ""
      if diff
        @debug "#{@name} publishing diff"
        @emit "publish", [@name, "", status, JSON.stringify(diff)]

    @sendData = (data, reset) =>
      # always reset and send full data. Diffing is deprecated
      reset = true

      data = helpers.objectify data

      unless reset
        @publishDiff data
      else
        @debug "Publishing full data"
        @cache = data #_.merge @cache, data
        @emit "publish", [@name, "", @status, JSON.stringify(data)]

    @sendCommand = (reqObject) =>
      {producer, action, args} = reqObject
      @info "Requesting data from #{producer} (#{action})"
      if args
        @emit "request", producer, action, JSON.stringify(args)
      else
        @emit "request", producer, action

    @sendError = (e) =>
      @error "error"
      @error e
      @emit "publish", [@name, e]

    @sendStatus = (status) =>
      @status = status or @status or ""
      @emit "publish", [@name, "", @status]

    @replyData = (data, callback) ->
      callback null, data

  subscribe: =>
    @debug "Setting subscription time"
    @subscriptionTime = new Date()

module.exports = Producer
